Option Explicit On 
Option Strict On
Option Compare Binary

Imports System.IO

' <remarks>
'   Eine einfache Klasse zum rekursiven 
'   Enumerieren von Dateien und Ordnern in
'   einem Dateisystem.
' </remarks>
Public Class FileSystemEnumerator
    Private m_StartDirectory As DirectoryInfo

    ' <summary>
    '   Wird ausgelst, wenn eine Datei gefunden wird.
    ' </summary>
    ' <param name="File">Gefundene Datei.</param>
    Public Event FileFound(ByVal File As FileInfo)

    ' <summary>
    '   Wird ausgelst, wenn ein Ordner gefunden wird.
    ' </summary>
    ' <param name="Directory">Gefundener Ordner.</param>
    ' <param name="ContinueRecursion">
    '   Boolescher Parameter, in dem angebenen wird, 
    '   ob die rekursive Enumeration fortgesetzt
    '   werden soll.
    ' </param>
    Public Event DirectoryFound( _
      ByVal Directory As DirectoryInfo, _
      ByRef ContinueRecursion As Boolean)

    ' <summary>
    '   Startet die Enumeration der Objekte im in 
    '   der Eigenschaft <c>StartDirectory</c>
    '   angegebenen Ordner.
    ' </summary>
    Public Sub Enumerate()
        CheckDirectory(m_StartDirectory)
    End Sub

    ' <summary>
    '   Gibt den Ordner, in dem die Enumeration 
    '   gestartet werden soll, an oder gibt ihn
    '   zurck.
    ' </summary>
    ' <value>Ordner, in dem die Enumeration 
    '   gestartet werden soll.</value>
    Public Property StartDirectory() As DirectoryInfo
        Get
            Return m_StartDirectory
        End Get
        Set(ByVal Value As DirectoryInfo)
            m_StartDirectory = Value
        End Set
    End Property

    ' <summary>
    '   Startet die Enumeration aller Dateien, die 
    '   direkt im Ordner liegen und geht dann
    '   alle Ordner, die direkt in einem Ordner 
    '   liegen, durch und lst fr jeden Ordner
    '   das Ereignis DirectoryFound aus. Dann wird 
    '   ggf. die Enumeration rekursiv auf enthaltene
    '   Ordner fortgesetzt.
    ' </summary>
    ' <param name="Directory">Ordner, dessen 
    '   Inhalt enumeriert werden soll.</param>
    Private Sub CheckDirectory( _
      ByVal Directory As DirectoryInfo)
        CheckFiles(Directory)
        Dim di As DirectoryInfo
        For Each di In Directory.GetDirectories()
            Dim b As Boolean = True
            RaiseEvent DirectoryFound(di, b)
            If b Then
                CheckDirectory(di)
            End If
        Next di
    End Sub

    ' <summary>
    '   Geht alle Dateien, die direkt in 
    '   einem Ordner liegen, durch und lst fr jede
    '   Datei das Ereignis FileFound aus.
    ' </summary>
    ' <param name="Directory">Ordner, dessen Dateien 
    '   enumeriert werden sollen.</param>
    Private Sub CheckFiles( _
      ByVal Directory As DirectoryInfo)
        Dim fi As FileInfo
        For Each fi In Directory.GetFiles()
            RaiseEvent FileFound(fi)
        Next fi
    End Sub
End Class